#!/usr/bin/env node
"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const program = require("commander");
const child_process_1 = require("child_process");
const fs = require("fs");
const config_1 = require("./source/config");
const express_extensions_1 = require("@codeplace/express-extensions");
const path = require("path");
const cliConfigsFileName = 'app.config.json';
program
    .command('create <type> <name> <package-name>')
    .option('-p, --list <items>', 'Packages to includes in creation', config_1.list)
    .action(function (type, packageRootFolder, packageName, cmd) {
    type = type.toLowerCase();
    const directoryExists = fs.existsSync(`./${packageRootFolder}`);
    const packages = cmd.list ? cmd.list : [];
    if (!directoryExists) {
        fs.mkdirSync(`./${packageRootFolder}`);
        console.log('Directory created.');
        fs.mkdirSync(`./${packageRootFolder}/source`);
        console.log('Source dir created.');
        fs.writeFileSync(`./${packageRootFolder}/package.json`, config_1.packageJson(type, packageName));
        console.log('package.json created.');
        fs.writeFileSync(`./${packageRootFolder}/tsconfig.json`, config_1.tsconfigJson());
        console.log('tsconfig.json created.');
        const buffer = child_process_1.execSync(`cd ./${packageRootFolder} && npm install typescript --save-dev`);
        process.stdout.write(buffer);
        packages.forEach((pack) => {
            const buffer = child_process_1.execSync(`cd ./${packageRootFolder} && npm install ${pack} --save`);
            process.stdout.write(buffer);
        });
        switch (type) {
            case 'api':
                child_process_1.exec(`cd ./${packageRootFolder} && npm install @codeplace/express-extensions --save`).stdout.pipe(process.stdout);
                fs.writeFileSync(`./${packageRootFolder}/index.ts`, `require('./source/server')`);
                fs.writeFileSync(`./${packageRootFolder}/source/server.ts`, `${config_1.apiConfig()}`);
                fs.writeFileSync(`./${packageRootFolder}/source/globalVars.ts`, `${config_1.globalVars()}`);
                fs.mkdirSync(`./${packageRootFolder}/source/controllers`);
                fs.mkdirSync(`./${packageRootFolder}/source/config`);
                fs.writeFileSync(`./${packageRootFolder}/source/config/loadControllers.ts`, `${express_extensions_1.LoadControllersScriptTemplate()}`);
                fs.writeFileSync(`./${packageRootFolder}/source/config/bootstrap.ts`, `${config_1.bootstrapFile()}`);
                fs.mkdirSync(`./${packageRootFolder}/source/routes`);
                fs.mkdirSync(`./${packageRootFolder}/source/models`);
                fs.mkdirSync(`./${packageRootFolder}/source/helpers`);
                fs.writeFileSync(`./${packageRootFolder}/source/helpers/Environment.ts`, `${config_1.environmentHelper()}`);
                fs.writeFileSync(`./${packageRootFolder}/source/helpers/index.ts`, `${config_1.helpersDirectoryIndex()}`);
                fs.mkdirSync(`./${packageRootFolder}/source/environments`);
                fs.writeFileSync(`./${packageRootFolder}/source/environments/current.env.json`, `${config_1.currentEnvironmentContent()}`);
                fs.writeFileSync(`./${packageRootFolder}/source/environments/local.env.json`, `${config_1.localEnvironmentContent()}`);
                fs.writeFileSync(path.resolve(process.cwd(), packageRootFolder, cliConfigsFileName), config_1.cliConfigTemplate());
                break;
            case 'lib':
                fs.writeFileSync(`./${packageRootFolder}/index.ts`, `export * from './source'`);
                break;
        }
    }
    else {
        console.log('A Directory with the specified name already exists');
    }
});
program
    .command('controller <name> <prefix> <version>')
    .action(function (name, prefix, version, cmd) {
    const cliConfigs = getCLIConfigs();
    const versionDirectory = `./${cliConfigs.controllersDirectory}/v${version}`;
    console.log(`Controller destination directory: ${versionDirectory}`);
    const directoryExists = fs.existsSync(versionDirectory);
    if (!directoryExists)
        fs.mkdirSync(versionDirectory);
    const controllerName = `${name}Controller.ts`;
    const controllerPath = path.resolve(versionDirectory, controllerName);
    if (fs.existsSync(controllerPath))
        return console.log(`${name}Controller already exists!`);
    fs.writeFileSync(controllerPath, express_extensions_1.ControllerTemplate(name, version, prefix));
    setTimeout(() => {
        runRoutesFilesCreator();
    }, 800);
});
program
    .command('routes')
    .action(function (cmd) {
    runRoutesFilesCreator();
});
program.parse(process.argv);
function runRoutesFilesCreator() {
    const cliConfigs = getCLIConfigs();
    const loadControllersScript = getLoadControllersScript(cliConfigs);
    const destinationDirectory = getRoutesDestinationDirectory(cliConfigs);
    callRoutesFilesCreator(destinationDirectory, loadControllersScript);
}
function getRoutesDestinationDirectory(cliConfigs) {
    const rootDirectory = process.cwd();
    const destinationDirectory = path.resolve(rootDirectory, cliConfigs.routesDirectory);
    console.log(`Routes destination directory: ${destinationDirectory}`);
    return destinationDirectory;
}
function getLoadControllersScript(cliConfigs) {
    const rootDirectory = process.cwd();
    const loadControllersScript = path.resolve(rootDirectory, cliConfigs.loadControllersScript);
    console.log(`Load controllers script: ${loadControllersScript}`);
    return loadControllersScript;
}
function callRoutesFilesCreator(destinationDirectory, loadControllersScript) {
    try {
        const instances = require(loadControllersScript);
        console.log(`Available controllers: ${Object.getOwnPropertyNames(instances)}`);
        express_extensions_1.CreateRoutesFiles(instances, destinationDirectory);
        console.log(`Routes created!`);
    }
    catch (error) {
        console.error(error.message);
    }
}
function getCLIConfigs() {
    const rootDirectory = process.cwd();
    const cliConfigsPath = path.resolve(rootDirectory, cliConfigsFileName);
    if (!fs.existsSync(cliConfigsPath)) {
        console.log('CLI configs not found!');
        process.exit();
    }
    const cliConfigs = require(cliConfigsPath);
    console.log('CLI configs loaded!');
    return cliConfigs;
}
//# sourceMappingURL=index.js.map